#!/usr/bin/perl -w
#===============================================================================
# $Header: /home/hiro/projects/dvr2/local/RCS/syncav.pl,v 1.2 2012/07/11 21:52:47 hiro Exp $
#         FILE:  syncav.pl
#        USAGE:  ./syncav.pl yyyymmddhhmmn [s]
#
#  DESCRIPTION:  Re-sync audio and video by placing a delay in audio stream
#
#       AUTHOR:  Copyright (c) 2011 Hiro Sugawara
#      VERSION:  1.0
#      CREATED:  10/31/2011 22:49:05
#     REVISION:  ---
#===============================================================================

use strict;
use warnings;

my $AVIROOT	= '/var/mm/video';
my $FFMPEG	= '/usr/local/bin/ffmpeg';

my $avifile	= shift @ARGV;
my $delay	= shift @ARGV;

unless (defined($avifile)) {
	print "Quick-n-Dirty tool for synchronizing A/V by adding audio delay\n";
	print "Usage: $0 yyyymmddhhmmn.avi [delay]\n";
	exit 0;
}

$delay = 4.5	unless defined($delay);
$avifile .= '.avi'	unless ($avifile =~ /\.avi$/);
$avifile = "$AVIROOT/$avifile"	unless ($avifile =~ /^\//);

my $origfile	= "$avifile.orig";
my $outfile	= "$avifile";

my $totalkbps;
my $videokbps;
my $videocodec;
my $audiokbps;
my $audiorate;
my $vtag = 'DIVX';
my $videostream = 0;
my $audiostream = 1;

( -e $avifile ) || ( -e $origfile ) || die "can't find $avifile\n";
my $inputfile = ( -e $origfile ) ? $origfile : $avifile;

open FFMPEG, "$FFMPEG -i $inputfile 2>&1 |"	|| die "can't do ffmpeg\n";
while (<FFMPEG>) {
	chomp;
	if (/Duration: .*, bitrate: (\d+)\s?kb\/s/) {
		$totalkbps	= $1;
	}
	if (/Stream #0.(\d).* Video: (\w+), /) {
		$videostream = $1;
		$videocodec	= $2;
	}
	if (/Stream #0.(\d).* Audio: .* (\d+)\s?Hz, .*, (\d+)\s?kb\/s/) {
		$audiostream = $1;
		$audiorate	= $2;
		$audiokbps	= $3;
	}
}
close FFMPEG;

if ($audiostream != 0) {
	$delay *= -1.0;
}

my $ffmpeg = sprintf
	"$FFMPEG -i %s -itsoffset %f -i %s -map 1:0 -map 0:1 " .
	"-vcodec %s -ar %d -ab %d -vb %d -vtag %s %s",
		$origfile, $delay, $origfile, $videocodec,
		$audiorate, $audiokbps * 1000, ($totalkbps - $audiokbps) * 1000, $vtag,
		$outfile;

print "$ffmpeg\n";

print STDERR "OK? ";
my $ans = <STDIN>;
chomp $ans;
($ans =~ /^y/i) || exit 0;

( -e $origfile ) || rename $avifile, $origfile;
system $ffmpeg;
